/**
 * copyright by andr stein.
 *  http://www.steinsoft.net
 *  mail: stone@steinsoft.net
 * 
 * Code is under no license and can be used
 * freely in any type of application.
 * 
 * In case of bugs or improvements ideas please free
 * to contact me.
 * 
 * Visit my website for other code.
 */
 
#ifndef S_VECTOR_HH
#define S_VECTOR_HH
 
#include <math.h>
#include <ostream>
 
 
/**
 * Class that generalizes a vector object with any type
 * of Scalar value and any type of dimension.
 * 
 * Use sVec3f oder sVec4f as abbreviation for example. Using
 * sVector<double,3> will create a vector of dimension 3 with
 * scalar type double.
 * @author Andr Stein
 */
template <class _scalar, int _dim>
class sVector {
public:
	sVector() { }
	sVector(const sVector<_scalar,_dim>& rhs) {
		for (int i = 0; i < _dim; ++i) data[i] = rhs.data[i];
	}
	
	sVector(_scalar s1, _scalar s2) { data[0] = s1; data[1] = s2; }
	sVector(_scalar s1, _scalar s2, _scalar s3) { data[0] = s1; data[1] = s2; data[2] = s3; }
	sVector(_scalar s1, _scalar s2, _scalar s3, _scalar s4) { data[0] = s1; data[1] = s2; data[2] = s3; data[3] = s4; }
	
	/**
	 * get/setters
	 */
	_scalar* getData() const {
		 return (_scalar*)data; 
	}
	
	void set(_scalar* data) {
		for (int i = 0; i < _dim; ++i) this->data[i] = data[i];
	}	
	
	const _scalar getLength() const {
		_scalar blf = 0;
		for (int i = 0; i <_dim; ++i) blf += data[i]*data[i];
		
		return sqrt(blf);
	}
	
	sVector<_scalar,_dim> getUnitVector() const {
		return (*this)/getLength();
	}
	
	_scalar dot(const sVector<_scalar,_dim>& rhs) const {
		_scalar s = 0;
		
		for (int i = 0; i < _dim; ++i) {
			s += data[i] * rhs.data[i];
		}
			
		return s;
	}
	
	sVector<_scalar,_dim> cross(const sVector<_scalar,_dim>& rhs) const {
		sVector<_scalar,_dim> rv;
		
		if (_dim == 2) {
			rv[0] = data[0]*rhs.data[1];
			rv[1] = data[1]*rhs.data[0];
		}
		else if (_dim == 3) {
			rv[0] = data[1]*rhs.data[2] - data[2]*rhs.data[1];
			rv[1] = data[2]*rhs.data[0] - data[0]*rhs.data[2];
			rv[2] = data[0]*rhs.data[1] - data[1]*rhs.data[0];
		}
		
		return rv;
	}
	
	double getAngle(const sVector<_scalar,_dim>& rhs) const {
		return acos(dot(rhs));
	}
	
	sVector<_scalar,_dim> getReflection(const sVector<_scalar,_dim>& rhs) const {
		const sVector<_scalar,_dim> v = getUnitVector();
		
		return ( v - rhs * 2.0f * ( v % rhs ) ) *
				getLength() ;
	}
	
	sVector<_scalar,_dim>& normalize() {
		(*this) /= getLength();
		return (*this);
	}
	
	sVector<_scalar,_dim> getNormal() const {
		return (*this) / getLength();
	}
	
	/**
	 * operators
	 */
	
	_scalar& operator [] (const int i) {
		return data[i];
	}
	
	sVector<_scalar,_dim>& operator = (const sVector<_scalar,_dim>& rhs) {
		for (int i = 0; i < _dim; ++i) data[i] = rhs.data[i];
		
		return *this;
	}
	
	sVector<_scalar,_dim>& operator = (_scalar* data) {
		set(data);
	}
	
	operator _scalar*() {
		return getData();
	}
	
	sVector<_scalar,_dim> operator + (const sVector<_scalar,_dim>& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]+rhs.data[i];
		
		return result;
	}
	
	sVector<_scalar,_dim> operator - (const sVector<_scalar,_dim>& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]-rhs.data[i];
		
		return result;
	}
	
	sVector<_scalar,_dim> operator * (const sVector<_scalar,_dim>& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]*rhs.data[i];
		
		return result;
	}
	
	sVector<_scalar,_dim> operator * (const _scalar& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]*rhs;
		
		return result;
	}
	
	
	sVector<_scalar,_dim> operator / (const sVector<_scalar,_dim>& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]/rhs.data[i];
		
		return result;
	}
		
	sVector<_scalar,_dim> operator / (const _scalar& rhs) const {
		sVector<_scalar,_dim> result;
		
		for (int i = 0; i < _dim; ++i) result.data[i] = data[i]/rhs;
		
		return result;
	}
	
	sVector<_scalar,_dim>& operator += (const sVector<_scalar,_dim>& rhs) {
		(*this) = operator + (rhs);
		return (*this);
	}
	
	sVector<_scalar,_dim>& operator -= (const sVector<_scalar,_dim>& rhs) {
		(*this) = operator - (rhs);
		return (*this);
	}
	
	sVector<_scalar,_dim>& operator *= (const sVector<_scalar,_dim>& rhs) {
		(*this) = operator * (rhs);
		return (*this);
	}
	
	sVector<_scalar,_dim>& operator *= (const _scalar& rhs) {
		(*this) = operator * (rhs);
		return (*this);
	}
	
	sVector<_scalar,_dim>& operator /= (const sVector<_scalar,_dim>& rhs) {
		(*this) = operator / (rhs);
		return (*this);
	}
	
	sVector<_scalar,_dim>& operator /= (const _scalar& rhs) {
		(*this) = operator / (rhs);
		return (*this);
	}
	
	bool operator == (const sVector<_scalar,_dim>& rhs) const {
		for (int i = 0; i < _dim; ++i) {
			if (data[i] != rhs.data[i]) return false;
		}
		
		return true;
	}
	
	sVector<_scalar,_dim> operator % (const sVector<_scalar,_dim>& rhs) const {
		return cross(rhs);
	}
	
	_scalar operator | (const sVector<_scalar,_dim>& rhs) const {
		return dot(rhs);
	}
	
	sVector<_scalar,_dim> operator -() const {
		sVector<_scalar,_dim> r;
		
		for (int i = 0; i < _dim; ++i) r[i] = -data[i];
		
		return r;
	}
		
private:
	_scalar data[_dim];	
};

// typedefs
typedef sVector<double,2> sVec2d;
typedef sVector<float,2> sVec2f;
typedef sVector<double,3> sVec3d;
typedef sVector<float,3> sVec3f;
typedef sVector<double,4> sVec4d;
typedef sVector<float,4> sVec4f;

// ostream operators
template <int _dim>
std::ostream& operator << (std::ostream& stream, sVector<float,_dim>& vector) {
	for (int i = 0; i < _dim; ++i) {
		stream << vector[i] << " ";
	}
	
	return stream;	
}

template <int _dim>
std::ostream& operator << (std::ostream& stream, sVector<double,_dim>& vector) {
	for (int i = 0; i < _dim; ++i) {
		stream << vector[i] << " ";
	}
	
	return stream;	
}

template <int _dim>
std::ostream& operator << (std::ostream& stream, sVector<int,_dim>& vector) {
	for (int i = 0; i < _dim; ++i) {
		stream << vector[i] << " ";
	}

	return stream;	
}
 
#endif


